INTER_AUTO: int = -1
INTER_NEAREST: int = 0
INTER_LINEAR: int = 1
INTER_CUBIC: int = 2
INTER_AREA: int = 3
INTER_LANCZOS4: int = 4
INTER_LANCZOS2: int = 10
INTER_LANCZOS3: int = 11
INTER_MITCHELL: int = 21
INTER_CATMULL_ROM: int = 22
INTER_B_SPLINE: int = 23
