from __future__ import annotations
import base64 as base64
import builtins as __builtins__
import pickle as pickle
__all__ = ['base64', 'load_emap', 'pickle']
def load_emap():
    ...
__test__: dict = {}
